;Just as a simple example: My own gauss approx. filter for
;images
;Don't forget to compile -> enormeous speed-up!
(defun smooth_pixel (arr i j)
       (setf (aref arr i j) 
       	     (/ (+ (* 4 (aref arr i       j      ))
       	     	   (* 2 (aref arr (+ i 1) j      ))
	     	   (* 2 (aref arr (- i 1) j      ))
	     	   (* 2 (aref arr i       (+ j 1)))
	     	   (* 2 (aref arr i       (- j 1)))
       	     	   (* 1 (aref arr (+ i 1) (+ j 1)))
	     	   (* 1 (aref arr (+ i 1) (- j 1)))
	     	   (* 1 (aref arr (- i 1) (+ j 1)))
	     	   (* 1 (aref arr (- i 1) (- j 1))))
               16)))
(compile 'smooth_pixel)
(defun mysmooth (arr)
      (do* ((j 1 (+ j 1)))
      	  ((= j (- (array-dimension arr 1) 1)))
	      (do* ((i 1 (+ i 1)))
      	      	  ((= i (- (array-dimension arr 0) 1)))
		      (smooth_pixel arr i j))))
(compile 'mysmooth)

;Always the first step: load the VIGRA CL Lib
(load "vigracl.cl")

(in-package :vigracl)

(print "loading blox-image")
(time (setq img (loadimage  "blox.gif")))

(print "performance test gaussian smoothing")
(print "vigra-method:")
(time (gsmooth img 0.3))
(print "my-own-method:")
;(time (mysmooth img))


(print "performing watershed transform on resized gradient image")
(time (setq img2  (labelimage
	    (watersheds
	     (ggradient 
	      (resizeimage img 800 800 4)
	      3.0)))))

(print "performing fft on image")
(time (setq img3 (fouriertransform img 800 800)))

(print "testing rotation and reflection functions on image")
(time (setq img4 (reflectimage img 3)))
(time (setq img5 (rotateimage img 15.0 3)))

(print "performing distance transform on canny edges of image")
(time (setq img6 (distancetransform
		  (cannyedgeimage img 1.8 0.1 100.0)
		  0.0 2)))

(print "testing difference of exponential edge detection on image")
(time (setq img7 (differenceofexponentialedgeimage img 0.8 0.1 100.0)))

(print "testing nearly all filters")
(time (setq img8 (gsmooth img 3.0)))
(time (setq img9 (laplacianofgaussian img 3.0)))
;(time (setq img10 (gsharpening img 0.5 3.0)))
;(time (setq img11 (sharpening img 3.0)))
;(time (setq img12 (nonlineardiffusion img 0.1 2.0)))

(print "saving resulting images")
(saveimage img2  "blox-relabeled-watersheds-on-resized-gradient-image.png")

(saveimage (car img3) "blox-fft-real.png")
(saveimage (cdr img3) "blox-fft-imag.png")

(saveimage img4  "blox-reflected-both.png")
(saveimage img5  "blox-rotated-15deg.png")
(saveimage img6  "blox-disttransform-on-canny.png")
(saveimage img7  "blox-diff_of_exp.png")
(saveimage img8  "blox-gsmooth-3.0.png")
(saveimage img9  "blox-log-3.0.png")
;(saveimage img10 "blox-gsharpening-0.5-3.0.png")
;(saveimage img11 "blox-sharpening-3.0.png")
;(saveimage img12 "blox-nonlineardiffusion-0.1-2.0.png")