#include "vigra/basicimageview.hxx"
#include "vigra/convolution.hxx"
#include "vigra/nonlineardiffusion.hxx" 
#include "vigra/distancetransform.hxx"
#include "os_settings.hxx"

LIBEXPORT int vigra_gaussiangradientmagnitude_c(const float *arr, const float *arr2,const  int width,const  int height,const  float sigma){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
            
      vigra::gaussianGradientMagnitude(srcImageRange(img), destImage(img2), sigma);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_gaussiansmoothing_c(const float *arr ,const float *arr2,const  int width,const  int height,const  float sigma){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
            
      vigra::gaussianSmoothing(srcImageRange(img), destImage(img2), sigma);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_laplacianofgaussian_c(const float *arr, const float *arr2,const  int width,const int height,const  float scale){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
            
      vigra::laplacianOfGaussian(srcImageRange(img), destImage(img2), scale);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_gaussiansharpening_c(const float *arr, const float *arr2,const int width, const int height, const float sharpening_factor, const float scale){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
            
      vigra::gaussianSharpening(srcImageRange(img), destImage(img2),sharpening_factor, scale);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_simplesharpening_c(const float *arr, const float *arr2, const int width, const int height, float sharpening_factor){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
            
      vigra::simpleSharpening(srcImageRange(img), destImage(img2),sharpening_factor);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_nonlineardiffusion_c(const float *arr, const float *arr2, const int width, const int height, const float edge_threshold, const float scale){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
      
      vigra::nonlinearDiffusion(srcImageRange(img), destImage(img2),
				vigra::DiffusivityFunctor<float>(edge_threshold), scale);
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}

LIBEXPORT int vigra_distancetransform_c(const float *arr, const float *arr2, const int width, const int height, const float background_label, const int norm){ 
    try
    {
      // create a gray scale image of appropriate size
      vigra::BasicImageView<float> img(arr, width, height);
      vigra::BasicImageView<float> img2(arr2, width, height);
      
      if(norm>=0 && norm <= 2)
        vigra::distanceTransform(   srcImageRange(img), destImage(img2),
                                    background_label,   norm);
      else
        return 2;
    }
    catch (vigra::StdException & e)
    {
        return 1;
    }
    return 0;
}


