(in-package vigracl.filters)

;###############################################################################
;###################          Gaussian Smoothing            ####################

(ff:def-foreign-call vigra_gaussiansmoothing_c 
    ((arr      (:array :float))
     (arr2      (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float))
  :returning  (:int fixnum))

(defun gsmooth (arr sigma)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiansmoothing_c arr arr2 width height sigma)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.gsmooth: Gaussian smoothing failed!"))))


;###############################################################################
;###################      Gaussian Gradient (Magnitude)     ####################

(ff:def-foreign-call vigra_gaussiangradientmagnitude_c
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sigma   :float))
  :returning  (:int fixnum))


(defun ggradient (arr sigma)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiangradientmagnitude_c arr arr2 width height sigma)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.ggradient: Gaussian gradient magnitude failed!"))))


;###############################################################################
;###################        Laplacian Of Gaussian           ####################

(ff:def-foreign-call vigra_laplacianofgaussian_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (scale   :float))
  :returning  (:int fixnum))

(defun laplacianofgaussian (arr scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_laplacianofgaussian_c arr arr2 width height scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.laplacianofgaussian: Laplacian of Gaussian failed!"))))



;###############################################################################
;###################          Gaussian Sharpening           ####################

(ff:def-foreign-call vigra_gaussiansharpening_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sharpening_factor   :float)
     (scale   :float))
  :returning  (:int fixnum))

(defun gsharpening (arr sharpening_factor scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_gaussiansharpening_c arr arr2 width height sharpening_factor scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.gsharpening: Gaussian sharpening failed!"))))


;###############################################################################
;###################            Simple Sharpening           ####################

(ff:def-foreign-call vigra_simplesharpening_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (sharpening_factor   :float))
  :returning  (:int fixnum))

(defun sharpening (arr sharpening_factor )
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			  :initial-element 0.0))
	 (foo   (vigra_simplesharpening_c arr arr2 width height sharpening_factor)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.sharpening: Simple sharpening failed!"))))


;###############################################################################
;###################          Nonlinear Diffusion           ####################

(ff:def-foreign-call vigra_nonlineardiffusion_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (edge_threshold   :float)
     (scale   :float))
  :returning  (:int fixnum))

(defun nonlineardiffusion (arr edge_threshold scale)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_nonlineardiffusion_c arr arr2 width height edge_threshold scale)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.nonlineardiffusion: Non linear Diffusion failed!!"))))

;###############################################################################
;###################          Distance Transform            ####################

(ff:def-foreign-call vigra_distancetransform_c 
    ((arr      (:array :float))
     (arr2     (:array :float))
     (width    :int fixnum) 
     (height   :int fixnum)
     (background_label   :float)
     (norm   :int))
  :returning  (:int fixnum))

(defun distancetransform (arr background_label norm)
  (let* ((width  (array-dimension arr 0))
	 (height (array-dimension arr 1))
	 (arr2 (make-array (array-dimensions arr)
			   :element-type (array-element-type arr)
			   :initial-element 0.0))
	 (foo   (vigra_distancetransform_c arr arr2 width height background_label norm)))
    (case foo
      ((0) arr2)
      ((1) "Error in vigracl.filters.distancetransform: Distance transformation failed!!")
      ((2) "Error in vigracl.filters.distancetransform: Norm must be in {0,1,2} !!"))))