(in-package :vigracl.helpers)

(defun array-copy (array)
  "Copies an array into a ewly allocated one -> fast."
  (let ((a (make-array (array-dimensions array) :adjustable nil)))
    (dotimes (i (array-total-size a) a)
      (setf (row-major-aref a i) (row-major-aref array i)))))
(compile 'array-copy)

(defun array-map (arr func)  
  "Maps a given function to each element of the array and returns a new array"
    (let* ((width  (array-dimension arr 0))
           (height (array-dimension arr 1))
           (arr2   (make-array (array-dimensions arr)
                    :element-type (array-element-type arr)
                    :initial-element 0.0))
           (foo     (do* ((j 0 (+ j 1)))
                         ((= j height))
                         (do* ((i 0 (+ i 1)))
                              ((= i width))
                                  (setf  (aref arr2 i j) (funcall func (aref arr i j)))))))
       arr2))
(compile 'array-map)

(defun array-combine (arr1 arr2 func)  
  "Combines two arrays to a new one using a given two argument function"
    (let* ((width  (array-dimension arr1 0))
           (height (array-dimension arr1 1))
           (arr   (make-array (array-dimensions arr1)
                    :element-type (array-element-type arr1)
                    :initial-element 0.0))
           (foo     (do* ((j 0 (+ j 1)))
                         ((= j height))
                         (do* ((i 0 (+ i 1)))
                              ((= i width))
                                  (setf  (aref arr i j) (funcall func (aref arr1 i j) (aref arr2 i j)))))))
       arr))
(compile 'array-combine)
